@@ -1,13 +1,13 @@
 /*
- * $Header: /home/cvspublic/jakarta-commons/logging/src/java/org/apache/commons/logging/LogFactory.java,v 1.11 2002/08/12 21:01:07 rsitze Exp $
- * $Revision: 1.11 $
- * $Date: 2002/08/12 21:01:07 $
+ * $Header: /home/cvspublic/jakarta-commons/logging/src/java/org/apache/commons/logging/LogFactory.java,v 1.21 2003/03/30 23:42:36 craigmcc Exp $
+ * $Revision: 1.21 $
+ * $Date: 2003/03/30 23:42:36 $
  *
  * ====================================================================
  *
  * The Apache Software License, Version 1.1
  *
- * Copyright (c) 1999-2002 The Apache Software Foundation.  All rights
+ * Copyright (c) 1999-2003 The Apache Software Foundation.  All rights
  * reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -62,30 +62,32 @@
 package org.apache.commons.logging;
 
 
-import java.io.InputStream;
-import java.io.IOException;
 import java.io.BufferedReader;
+import java.io.IOException;
+import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
+import java.security.AccessController;
+import java.security.PrivilegedAction;
 import java.util.Enumeration;
 import java.util.Hashtable;
 import java.util.Properties;
-import java.lang.SecurityException;
 
 
 /**
  * <p>Factory for creating {@link Log} instances, with discovery and
  * configuration features similar to that employed by standard Java APIs
  * such as JAXP.</p>
- *
+ * 
  * <p><strong>IMPLEMENTATION NOTE</strong> - This implementation is heavily
  * based on the SAXParserFactory and DocumentBuilderFactory implementations
  * (corresponding to the JAXP pluggability APIs) found in Apache Xerces.</p>
  *
  * @author Craig R. McClanahan
  * @author Costin Manolache
- * @version $Revision: 1.11 $ $Date: 2002/08/12 21:01:07 $
+ * @author Richard A. Sitze
+ * @version $Revision: 1.21 $ $Date: 2003/03/30 23:42:36 $
  */
 
 public abstract class LogFactory {
@@ -101,7 +103,6 @@ public abstract class LogFactory {
     public static final String FACTORY_PROPERTY =
         "org.apache.commons.logging.LogFactory";
 
-
     /**
      * The fully qualified class name of the fallback <code>LogFactory</code>
      * implementation class to use, if no other can be found.
@@ -109,7 +110,6 @@ public abstract class LogFactory {
     public static final String FACTORY_DEFAULT =
         "org.apache.commons.logging.impl.LogFactoryImpl";
 
-
     /**
      * The name of the properties file to search for.
      */
@@ -238,9 +238,10 @@ public abstract class LogFactory {
      * <ul>
      * <li>The <code>org.apache.commons.logging.LogFactory</code> system
      *     property.</li>
+     * <li>The JDK 1.3 Service Discovery mechanism</li>
      * <li>Use the properties file <code>commons-logging.properties</code>
      *     file, if found in the class path of this class.  The configuration
-     *     file is in standard <code>java.util.Propertis</code> format and
+     *     file is in standard <code>java.util.Properties</code> format and
      *     contains the fully qualified name of the implementation class
      *     with the key being the system property defined above.</li>
      * <li>Fall back to a default implementation class
@@ -258,13 +259,38 @@ public abstract class LogFactory {
     public static LogFactory getFactory() throws LogConfigurationException {
 
         // Identify the class loader we will be using
-        ClassLoader contextClassLoader = getContextClassLoader();
+        ClassLoader contextClassLoader =
+            (ClassLoader)AccessController.doPrivileged(
+                new PrivilegedAction() {
+                    public Object run() {
+                        return getContextClassLoader();
+                    }
+                });
 
         // Return any previously registered factory for this class loader
         LogFactory factory = getCachedFactory(contextClassLoader);
         if (factory != null)
             return factory;
 
+
+        // Load properties file..
+        // will be used one way or another in the end.
+
+        Properties props=null;
+        try {
+            InputStream stream = getResourceAsStream(contextClassLoader,
+                                                     FACTORY_PROPERTIES);
+
+            if (stream != null) {
+                props = new Properties();
+                props.load(stream);
+                stream.close();
+            }
+        } catch (IOException e) {
+        } catch (SecurityException e) {
+        }
+
+        
         // First, try the system property
         try {
             String factoryClass = System.getProperty(FACTORY_PROPERTY);
@@ -275,6 +301,7 @@ public abstract class LogFactory {
             ;  // ignore
         }
 
+
         // Second, try to find a service by using the JDK1.3 jar
         // discovery mechanism. This will allow users to plug a logger
         // by just placing it in the lib/ directory of the webapp ( or in
@@ -283,9 +310,8 @@ public abstract class LogFactory {
 
         if (factory == null) {
             try {
-                InputStream is = (contextClassLoader == null
-                                  ? ClassLoader.getSystemResourceAsStream( SERVICE_ID )
-                                  : contextClassLoader.getResourceAsStream( SERVICE_ID ));
+                InputStream is = getResourceAsStream(contextClassLoader,
+                                                     SERVICE_ID);
 
                 if( is != null ) {
                     // This code is needed by EBCDIC and other strange systems.
@@ -312,8 +338,6 @@ public abstract class LogFactory {
         }
 
 
-        Properties props=null;
-        
         // Third try a properties file. 
         // If the properties file exists, it'll be read and the properties
         // used. IMHO ( costin ) System property and JDK1.3 jar service
@@ -322,27 +346,16 @@ public abstract class LogFactory {
         // the webapp, even if a default logger is set at JVM level by a
         // system property )
 
-        try {
-            InputStream stream =
-                contextClassLoader.getResourceAsStream(FACTORY_PROPERTIES);
-            if (stream != null) {
-                props = new Properties();
-                props.load(stream);
-                stream.close();
-                String factoryClass = props.getProperty(FACTORY_PROPERTY);
-                if( factory==null ) {
-                    if (factoryClass == null) {
-                        factoryClass = FACTORY_DEFAULT;
-                    }
-                    factory = newFactory(factoryClass, contextClassLoader);
-                }
+        if (factory == null  &&  props != null) {
+            String factoryClass = props.getProperty(FACTORY_PROPERTY);
+            if (factoryClass != null) {
+                factory = newFactory(factoryClass, contextClassLoader);
             }
-            // the properties will be set at the end.
-        } catch (IOException e) {
-        } catch (SecurityException e) {
         }
 
+
         // Fourth, try the fallback implementation class
+
         if (factory == null) {
             factory = newFactory(FACTORY_DEFAULT, LogFactory.class.getClassLoader());
         }
@@ -352,14 +365,14 @@ public abstract class LogFactory {
              * Always cache using context class loader..
              */
             cacheFactory(contextClassLoader, factory);
-        }
 
-        if( props!=null ) {
-            Enumeration names = props.propertyNames();
-            while (names.hasMoreElements()) {
-                String name = (String) names.nextElement();
-                String value = props.getProperty(name);
-                factory.setAttribute(name, value);
+            if( props!=null ) {
+                Enumeration names = props.propertyNames();
+                while (names.hasMoreElements()) {
+                    String name = (String) names.nextElement();
+                    String value = props.getProperty(name);
+                    factory.setAttribute(name, value);
+                }
             }
         }
         
@@ -403,6 +416,27 @@ public abstract class LogFactory {
     }
 
 
+    /**
+     * Release any internal references to previously created {@link LogFactory}
+     * instances that have been associated with the specified class loader
+     * (if any), after calling the instance method <code>release()</code> on
+     * each of them.
+     *
+     * @param classLoader ClassLoader for which to release the LogFactory
+     */
+    public static void release(ClassLoader classLoader) {
+
+        synchronized (factories) {
+            LogFactory factory = (LogFactory) factories.get(classLoader);
+            if (factory != null) {
+                factory.release();
+                factories.remove(classLoader);
+            }
+        }
+
+    }
+
+
     /**
      * Release any internal references to previously created {@link LogFactory}
      * instances, after calling the instance method <code>release()</code> on
@@ -497,9 +531,6 @@ public abstract class LogFactory {
         
         if (contextClassLoader != null)
             factory = (LogFactory) factories.get(contextClassLoader);
-            
-        if (factory==null)
-            factory = (LogFactory) factories.get(LogFactory.class.getClassLoader());
         
         return factory;
     }
@@ -523,35 +554,81 @@ public abstract class LogFactory {
      * @exception LogConfigurationException if a suitable instance
      *  cannot be created
      */
-    protected static LogFactory newFactory(String factoryClass,
-                                           ClassLoader classLoader)
+    protected static LogFactory newFactory(final String factoryClass,
+                                           final ClassLoader classLoader)
         throws LogConfigurationException
     {
-        
-        try {
-            if (classLoader == null)
-                classLoader = LogFactory.class.getClassLoader();
-
-            Class clazz = null;
-            try {
-                // first the thread class loader
-                clazz = classLoader.loadClass(factoryClass);
-            } catch (ClassNotFoundException ex) {
-                // if this failed (i.e. no implementation is
-                // found in the webapp), try the caller's loader
-                // if we haven't already...
-                if (classLoader != LogFactory.class.getClassLoader()) {
-                    classLoader = LogFactory.class.getClassLoader();
-                    clazz = classLoader.loadClass(factoryClass);
+        Object result = AccessController.doPrivileged(
+            new PrivilegedAction() {
+                public Object run() {
+                    try {
+                        if (classLoader != null) {
+                            try {
+                                // first the given class loader param (thread class loader)
+                                
+                                // warning: must typecast here & allow exception
+                                // to be generated/caught & recast propertly.
+                                return (LogFactory)classLoader.loadClass(factoryClass).newInstance();
+                            } catch (ClassNotFoundException ex) {
+                                if (classLoader == LogFactory.class.getClassLoader()) {
+                                    // Nothing more to try, onwards.
+                                    throw ex;
+                                }
+                                // ignore exception, continue
+                            } catch (NoClassDefFoundError e) {
+                                if (classLoader == LogFactory.class.getClassLoader()) {
+                                    // Nothing more to try, onwards.
+                                    throw e;
+                                }
+                              
+                            }catch(ClassCastException e){
+                                
+                              if (classLoader == LogFactory.class.getClassLoader()) {
+                                    // Nothing more to try, onwards (bug in loader implementation).
+                                    throw e;
+                               }
+                            }
+                            // ignore exception, continue  
+                        }
+            
+                        /* At this point, either classLoader == null, OR
+                         * classLoader was unable to load factoryClass..
+                         * try the class loader that loaded this class:
+                         * LogFactory.getClassLoader().
+                         * 
+                         * Notes:
+                         * a) LogFactory.class.getClassLoader() may return 'null'
+                         *    if LogFactory is loaded by the bootstrap classloader.
+                         * b) The Java endorsed library mechanism is instead
+                         *    Class.forName(factoryClass);
+                         */
+                        // warning: must typecast here & allow exception
+                        // to be generated/caught & recast propertly.
+                        return (LogFactory)Class.forName(factoryClass).newInstance();
+                    } catch (Exception e) {
+                        return new LogConfigurationException(e);
+                    }
                 }
-            }
-            
-            LogFactory factory = (LogFactory)clazz.newInstance();
-            
-            return factory;
-        } catch (Exception e) {
-            throw new LogConfigurationException(e);
-        }
+            });
 
+        if (result instanceof LogConfigurationException)
+            throw (LogConfigurationException)result;
+            
+        return (LogFactory)result;
+    }
+    
+    private static InputStream getResourceAsStream(final ClassLoader loader,
+                                                   final String name)
+    {
+        return (InputStream)AccessController.doPrivileged(
+            new PrivilegedAction() {
+                public Object run() {
+                    if (loader != null) {
+                        return loader.getResourceAsStream(name);
+                    } else {
+                        return ClassLoader.getSystemResourceAsStream(name);
+                    }
+                }
+            });
     }
 }
